@@ -19,7 +19,7 @@ def get_recursive_filelist(pathlist: List[Path]) -> List[Path]:
     for path_str in pathlist:
         path = Path(path_str)
         if path.is_dir():
-            for format in ["*.cbz", "*.cb7"]:
+            for format in ["*.cbz", "*.cb7", "*.cbr"]:
                 filelist.extend(iter(path.rglob(format)))
         else:
             filelist.append(path)
